unit untGoogleMap;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs,ActiveX, SHDocVw, MSHTML, Interfaces, Automation, OleCtrls, StdCtrls;

type
  TWebBrowser = class(SHDocVw.TWebBrowser, IDocHostUIHandler)
  private
    function GetHTMLWindow2: IHTMLWindow2;
    function GetDocument2: IHTMLDocument2;
  protected
    { IDocHostUIHandler }
    function EnableModeless(fEnable: Integer): HResult; stdcall;
    function FilterDataObject(const pDO: IDataObject; out ppDORet: IDataObject): HResult; stdcall;
    function GetDropTarget(const pDropTarget: IDropTarget; out ppDropTarget: IDropTarget): HResult; stdcall;
    function GetExternal(out ppDispatch: IDispatch): HResult; stdcall;
    function GetHostInfo(var pInfo: _DOCHOSTUIINFO): HResult; stdcall;
    function GetOptionKeyPath(out pchKey: PWideChar; dw: UINT): HResult; stdcall;
    function HideUI: HResult; stdcall;
    function OnDocWindowActivate(fActivate: Integer): HResult; stdcall;
    function OnFrameWindowActivate(fActivate: Integer): HResult; stdcall;
    function ResizeBorder(var prcBorder: tagRECT; const pUIWindow: IOleInPlaceUIWindow;
      fRameWindow: Integer): HResult; stdcall;
    function ShowContextMenu(dwID: UINT; ppt: PtagPOINT; const pcmdtReserved: IUnknown;
       const pdispReserved: IDispatch): HResult; stdcall;
    function ShowUI(dwID: UINT; const pActiveObject: IOleInPlaceActiveObject;
      const pCommandTarget: IOleCommandTarget; const pFrame: IOleInPlaceFrame;
      const pDoc: IOleInPlaceUIWindow): HResult; stdcall;
    function TranslateAccelerator(var lpmsg: tagMSG; var pguidCmdGroup: TGUID; nCmdID: UINT): HResult; stdcall;
    function TranslateUrl(dwTranslate: UINT; pchURLIn: PWideChar; out ppchURLOut: PWideChar): HResult; stdcall;
    function UpdateUI: HResult; stdcall;
  public
    property Document2: IHTMLDocument2 read GetDocument2;
    property HTMLWindow2: IHTMLWindow2 read GetHTMLWindow2;
  end;

  TApplicationWrapper = class(TObjectWrapper)
  private
    function GetApplication: TApplication;
  published
    procedure DoError(ErrorMessage : String);
    procedure DoGeoCodeFound(lat, lng : Double);
    procedure DoClick(lat, lng : Double);
    procedure DoMarkerClick(lat, lng : Double);

    property Application: TApplication read GetApplication;
  end;

  TfrmGoogleMap = class(TForm)
    btnLoad: TButton;
    wBrowser: TWebBrowser;
    btnFindTopkapi: TButton;
    btnAddMarker: TButton;
    btnGotoCoordinat: TButton;
    edtLat: TEdit;
    edtLng: TEdit;
    procedure btnLoadClick(Sender: TObject);
    procedure btnFindTopkapiClick(Sender: TObject);
    procedure btnAddMarkerClick(Sender: TObject);
    procedure btnGotoCoordinatClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  frmGoogleMap: TfrmGoogleMap;

  MarkerLat,
  MarkerLng : Double;

implementation

{$R *.dfm}

const
  Enter             = Char(13) + Char(10);
  HtmlFirstSection  = '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" ' + Enter +
                      '"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">';

  HtmlStartSection  = '<html xmlns="http://www.w3.org/1999/xhtml" xmlns:v="urn:schemas-microsoft-com:vml">';
  HtmlEndSection    = '</html>';
  HeadStartSection  = '<head>';
  HeadEndSection    = '</head>';

  BodyStartSection  = '<body onload="GoogleMap.CreateMap()" onunload="GUnload()">';
  BodyEndSection    = '</body>';
  BodyGoogle        = '<div id="Canvas" style="width: 500px; height: 300px"></div>';

  ScriptGoogle      = '<script src="http://maps.google.com/maps?file=api&amp;v=2&amp;sensor=true&amp;key=YOUR_GOOGLE_MAPS_KEY_HERE" '+
                      'type="text/javascript"></script>';

  ScriptStartSection= '<script type="text/javascript">';
  ScriptEndSection  = '</script>';
  MetaSection       = '<meta http-equiv="content-type" content="text/html; charset=utf-8"/>';


  HTML = HtmlFirstSection         + Enter +
         HtmlStartSection         + Enter +
          HeadStartSection        + Enter +
            MetaSection           + Enter +
            ScriptGoogle          + Enter +
            '%HEADSECTION%'       + Enter +
            '%SCRIPTSECTION%'     + Enter +
          HeadEndSection          + Enter +
          BodyStartSection        + Enter +
            BodyGoogle            + Enter +
            '%BODYSECTION%'       + Enter +
          BodyEndSection          + Enter +
         HtmlEndSection;


procedure LoadHTML(WebBrowser: TWebBrowser; HTMLCode: String);
var
  sl: TStringList;
  ms: TMemoryStream;
begin
  WebBrowser.Navigate('about:blank') ;

  if Assigned(WebBrowser.Document) then
  begin
     sl := TStringList.Create;
     try
        ms := TMemoryStream.Create;
        try
           sl.Text := HTMLCode;
           sl.SaveToStream(ms);
           ms.Seek(0, 0);
           (WebBrowser.Document as IPersistStreamInit).Load(TStreamAdapter.Create(ms));
        finally
           ms.Free;
        end;
     finally
        sl.Free;
     end;
  end;
end;

const
  JSGoogle =
   ' function TGoogleMap() ' + Enter +
   ' { ' + Enter +
   '     var map = null; ' + Enter +
   '     var smallMapControl = null; ' + Enter +
   '     var smallZoomControl= null; ' + Enter +

   '     this.CreateMap = CreateMap; ' + Enter +
   '     this.Render = Render; ' + Enter +
   '     this.FindGeoCode = FindGeoCode; ' + Enter +
   '     this.AddMarker = AddMarker; ' + Enter +
   '     this.OpenInfoWindow = OpenInfoWindow; ' + Enter +
   '     this.CloseInfoWindow = CloseInfoWindow; ' + Enter +
   '     this.AddSmallMapControl = AddSmallMapControl; ' + Enter +
   '     this.AddSmallZoomControl = AddSmallZoomControl; ' + Enter +
   '     this.RemoveSmallMapControl = RemoveSmallMapControl; ' + Enter +
   '     this.RemoveSmallZoomControl = RemoveSmallZoomControl; ' + Enter +
   '     this.GotoCoordinat = GotoCoordinat; ' + Enter +

   '     function CreateMap() ' + Enter +
   '     { ' + Enter +
   '         map = new GMap2(document.getElementById("Canvas")); ' + Enter +

   '         GEvent.addListener(map, "click", function(overlay, point, overlaypoint) { if(point) external.DoClick(point.lat(), point.lng()); } ); ' + Enter +
   '     } ' + Enter +

   '     function Render(lat, lng) ' + Enter +
   '     { ' + Enter +
   '         map.setCenter(new GLatLng(lat, lng), 15); ' + Enter +
   '     } ' + Enter +

   '     function FindGeoCode(address) ' + Enter +
   '     { ' + Enter +
   '         var geoCoder = new GClientGeocoder(); ' + Enter +
   '         geoCoder.getLatLng(address, ' + Enter +
   '                                     function(point) ' + Enter +
   '                                     { ' + Enter +
   '                                         if (!point) { external.DoError("Belirtilen adres bulunamadi.!"); } ' + Enter +
   '                                         else ' + Enter +
   '                                         { ' + Enter +
   '                                             external.DoGeoCodeFound(point.lat(), point.lng()); ' + Enter +
   '                                         } ' + Enter +
   '                                     } ' + Enter +
   '                            ); ' + Enter +
   '     } ' + Enter +

   '     function AddMarker(lat, lng, draggable) ' + Enter +
   '     { ' + Enter +
   '         var marker; ' + Enter +

   '         if (draggable) ' + Enter +
   '         { ' + Enter +
   '             marker = new GMarker(new GLatLng(lat, lng), {draggable: true}); ' + Enter +
   '         } ' + Enter +
   '         else ' + Enter +
   '         { ' + Enter +
   '             marker = new GMarker(new GLatLng(lat, lng), {draggable: false}); ' + Enter +
   '         } ' + Enter +

   '         map.addOverlay(marker); ' + Enter +
   '         GEvent.addListener(marker, "click", function() {' +
   '              var coord = marker.getLatLng();' + Enter +
   '              if(coord)' + Enter +
   '                 external.DoMarkerClick(coord.lat(), coord.lng()); } );' + Enter +
   '     } ' + Enter +

   '     function OpenInfoWindow(lat, lng, message) ' + Enter +
   '     { ' + Enter +
   '         map.openInfoWindow(new GLatLng(lat, lng), ' + Enter +
   '                            document.createTextNode(message)); ' + Enter +
   '     } ' + Enter +

   '     function CloseInfoWindow() ' + Enter +
   '     { ' + Enter +
   '         map.closeInfoWindow(); ' + Enter +
   '     } ' + Enter +

   '     function AddSmallMapControl() ' + Enter +
   '     { ' + Enter +
   '         if (!smallMapControl) ' + Enter +
   '         { ' + Enter +
   '             smallMapControl = new GSmallMapControl(); ' + Enter +
   '             map.addControl(smallMapControl); ' + Enter +
   '         } ' + Enter +
   '     } ' + Enter +

   '     function AddSmallZoomControl() ' + Enter +
   '     { ' + Enter +
   '         if (!smallZoomControl) ' + Enter +
   '         { ' + Enter +
   '             smallZoomControl = new GSmallZoomControl(); ' + Enter +
   '             map.addControl(smallZoomControl); ' + Enter +
   '         } ' + Enter +
   '     } ' + Enter +

   '     function RemoveSmallMapControl() ' + Enter +
   '     { ' + Enter +
   '         if (smallMapControl) ' + Enter +
   '         { ' + Enter +
   '             map.removeControl(smallMapControl); ' + Enter +
   '             smallMapControl = null; ' + Enter +
   '         } ' + Enter +
   '     } ' + Enter +

   '     function RemoveSmallZoomControl() ' + Enter +
   '     { ' + Enter +
   '         if (smallZoomControl) ' + Enter +
   '         { ' + Enter +
   '             map.removeControl(smallZoomControl); ' + Enter +
   '             smallZoomControl = null; ' + Enter +
   '         } ' + Enter +
   '     } ' + Enter +

   '     function GotoCoordinat(lat, lng) ' + Enter +
   '     { ' + Enter +
   '         map.panTo(new GLatLng(lat, lng)); ' + Enter +
   '     } ' + Enter +
   ' } ' + Enter +

   ' var GoogleMap = new TGoogleMap(); ';



(*
  'function TGoogleMap()' + Enter +
              '{' + Enter +
              ' var map = null;' + Enter +

              ' this.Render = Render;' + Enter +
              ' this.FindGeoCode = FindGeoCode;'+ Enter +
              ' this.CreateMap = CreateMap;' + Enter +

              '   function CreateMap()' + Enter +
              '   {' + Enter +
              '        map = new GMap2(document.getElementById("Canvas"));' + Enter +
              '   }' + Enter +

              '    function Render(lat, lng)' + Enter +
              '    {' + Enter +
              '        map.setCenter(new GLatLng(lat, lng), 13);' + Enter +

              '    }' + Enter +

              '    function FindGeoCode(address)' + Enter +
              '    {' + Enter +
              '        var geoCoder = new GClientGeocoder();' + Enter +
              '        geoCoder.getLatLng(address,' + Enter +
              '                                    function(point)' + Enter +
              '                                    {' + Enter +
              '                                        if (!point) { external.Error("Belirtilen adres bulunamadi."); } ' + Enter +
              '                                        else' + Enter +
              '                                        {' + Enter +
              '                                            external.GeoCodeFound(point.lat(), point.lng());' + Enter +
              '                                        }' + Enter +
              '                                    }' + Enter +
              '                           );' + Enter +
              '    }' + Enter +
              '}' + Enter +

              'var GoogleMap = new TGoogleMap();';
  *)


// external.Error("Belirtilen adres bulunamadi.");
//external.Error("Tarayiciniz Google Map ile uyumlu degil.!");

{ TWebBrowser }
{$REGION 'TWebBrowser Snf'}
const
  // Constants from mshtmhst.h
  DOCHOSTUIFLAG_DIALOG    = $1;
  DOCHOSTUIFLAG_DISABLE_HELP_MENU = $2;
  DOCHOSTUIFLAG_NO3DBORDER        = $4;
  DOCHOSTUIFLAG_SCROLL_NO = $8;
  DOCHOSTUIFLAG_DISABLE_SCRIPT_INACTIVE   = $10;
  DOCHOSTUIFLAG_OPENNEWWIN        = $20;
  DOCHOSTUIFLAG_DISABLE_OFFSCREEN = $40;
  DOCHOSTUIFLAG_FLAT_SCROLLBAR    = $80;
  DOCHOSTUIFLAG_DIV_BLOCKDEFAULT  = $100;
  DOCHOSTUIFLAG_ACTIVATE_CLIENTHIT_ONLY   = $200;
  DOCHOSTUIFLAG_OVERRIDEBEHAVIORFACTORY   = $400;
  DOCHOSTUIFLAG_CODEPAGELINKEDFONTS       = $800;
  DOCHOSTUIFLAG_URL_ENCODING_DISABLE_UTF8 = $1000;
  DOCHOSTUIFLAG_URL_ENCODING_ENABLE_UTF8  = $2000;
  DOCHOSTUIFLAG_ENABLE_FORMS_AUTOCOMPLETE = $4000;
  DOCHOSTUIFLAG_ENABLE_INPLACE_NAVIGATION = $10000;
  DOCHOSTUIFLAG_IME_ENABLE_RECONVERSION   = $20000;
  DOCHOSTUIFLAG_THEME     = $40000;
  DOCHOSTUIFLAG_NOTHEME   = $80000;
  DOCHOSTUIFLAG_NOPICS    = $100000;
  DOCHOSTUIFLAG_NO3DOUTERBORDER   = $200000;
  DOCHOSTUIFLAG_DISABLE_EDIT_NS_FIXUP	= $400000;
  DOCHOSTUIFLAG_LOCAL_MACHINE_ACCESS_CHECK	= $800000;
  DOCHOSTUIFLAG_DISABLE_UNTRUSTEDPROTOCOL	= $1000000;
  DOCHOSTUIFLAG_ENABLE_ACTIVEX_BLOCK_MODE = $2000000;
  DOCHOSTUIFLAG_ENABLE_ACTIVEX_PROMPT_MODE = $4000000;
  DOCHOSTUIFLAG_ENABLE_ACTIVEX_DEFAULT_MODE = $8000000;

function TWebBrowser.EnableModeless(fEnable: Integer): HResult;
begin
  Result := E_NOTIMPL;
end;

function TWebBrowser.FilterDataObject(const pDO: IDataObject;
  out ppDORet: IDataObject): HResult;
begin
  Result := E_NOTIMPL;
end;

function TWebBrowser.GetDocument2: IHTMLDocument2;
begin
  Supports(Document, IHTMLDocument2, Result);
end;

function TWebBrowser.GetDropTarget(const pDropTarget: IDropTarget;
  out ppDropTarget: IDropTarget): HResult;
begin
  Result := E_NOTIMPL;
end;

function TWebBrowser.GetExternal(out ppDispatch: IDispatch): HResult;
var
  W: TApplicationWrapper;
begin
  W := TApplicationWrapper.Connect(Forms.Application);
  ppDispatch := TAutoObjectDispatch.Create(W) as IDispatch;
  Result := S_OK;
end;

function TWebBrowser.GetHostInfo(var pInfo: _DOCHOSTUIINFO): HResult;
begin
  FillChar(pInfo, SizeOf(pInfo), 0);
  pInfo.cbSize := SizeOf(_DOCHOSTUIINFO);
  pInfo.dwFlags := DOCHOSTUIFLAG_SCROLL_NO or DOCHOSTUIFLAG_NO3DBORDER or DOCHOSTUIFLAG_ENABLE_FORMS_AUTOCOMPLETE or DOCHOSTUIFLAG_THEME;
  pInfo.dwDoubleClick := 0;
  Result := S_OK;
end;

function TWebBrowser.GetHTMLWindow2: IHTMLWindow2;
begin
  Result := Document2.parentWindow;
end;

function TWebBrowser.GetOptionKeyPath(out pchKey: PWideChar; dw: UINT): HResult;
begin
  Result := E_NOTIMPL;
end;

function TWebBrowser.HideUI: HResult;
begin
  Result := E_NOTIMPL;
end;

function TWebBrowser.OnDocWindowActivate(fActivate: Integer): HResult;
begin
  Result := E_NOTIMPL;
end;

function TWebBrowser.OnFrameWindowActivate(fActivate: Integer): HResult;
begin
  Result := E_NOTIMPL;
end;

function TWebBrowser.ResizeBorder(var prcBorder: tagRECT;
  const pUIWindow: IOleInPlaceUIWindow; fRameWindow: Integer): HResult;
begin
  Result := S_OK;
end;

function TWebBrowser.ShowContextMenu(dwID: UINT; ppt: PtagPOINT;
  const pcmdtReserved: IInterface; const pdispReserved: IDispatch): HResult;
begin
  Result := E_NOTIMPL;
end;

function TWebBrowser.ShowUI(dwID: UINT;
  const pActiveObject: IOleInPlaceActiveObject;
  const pCommandTarget: IOleCommandTarget; const pFrame: IOleInPlaceFrame;
  const pDoc: IOleInPlaceUIWindow): HResult;
begin
  Result := S_FALSE;
end;

function TWebBrowser.TranslateAccelerator(var lpmsg: tagMSG;
  var pguidCmdGroup: TGUID; nCmdID: UINT): HResult;
begin
  Result := E_NOTIMPL;
end;

function TWebBrowser.TranslateUrl(dwTranslate: UINT; pchURLIn: PWideChar;
  out ppchURLOut: PWideChar): HResult;
begin
  ppchURLOut := nil;
  Result := S_FALSE;
end;

function TWebBrowser.UpdateUI: HResult;
begin
  Result := S_FALSE;
end;
{$ENDREGION}

{ TApplicationWrapper }

procedure TApplicationWrapper.DoClick(lat, lng: Double);
begin
  frmGoogleMap.Caption := 'Map Clicked. Lat:' + FloatToStr(lat) + ' / Lng:' + FloatToStr(lng);
end;

procedure TApplicationWrapper.DoMarkerClick(lat, lng: Double);
begin
  frmGoogleMap.Caption := 'Marker Clicked. Lat:' + FloatToStr(lat) + ' / Lng:' + FloatToStr(lng);
end;

procedure TApplicationWrapper.DoError(ErrorMessage: String);
begin
  ShowMessage('Hata olutu:' + ErrorMessage);
end;

procedure TApplicationWrapper.DoGeoCodeFound(lat, lng: Double);
var
  Doc : IHTMLDocument2;
  Win : IHTMLWindow2;
  sLat,
  sLng : String;
begin
  ShowMessage('Geocode found. Lat=' + FloatToStr(lat) + ', Lng=' + FloatToStr(lng));

  sLat := FloatToStr(lat);
  sLng := FloatToStr(lng);

  sLat := StringReplace(sLat, ',', '.', [rfReplaceAll]);
  sLng := StringReplace(sLng, ',', '.', [rfReplaceAll]);

  Doc := frmGoogleMap.wBrowser.Document as IHTMLDocument2;
  Win := Doc.parentWindow;
  Win.execScript('GoogleMap.Render(' + sLat + ',' + sLng + ')', 'JavaScript');

  MarkerLat := lat;
  MarkerLng := lng;
end;

function TApplicationWrapper.GetApplication: TApplication;
begin
  Result := Forms.Application;
end;

procedure TfrmGoogleMap.btnLoadClick(Sender: TObject);
var
  sHTML,
  sScript : String;
begin
  sHTML := HTML;
  sScript :=  ScriptStartSection + Enter +
                JSGoogle + Enter +
              ScriptEndSection;

  sHTML := StringReplace(sHTML, '%HEADSECTION%'   , ''      , [rfReplaceAll]);
  sHTML := StringReplace(sHTML, '%SCRIPTSECTION%' , sScript , [rfReplaceAll]);
  sHTML := StringReplace(sHTML, '%BODYSECTION%'   , ''      , [rfReplaceAll]);

//  with TStringList.Create do
//  begin
//    Add(sHTML);
//    SaveToFile('c:\GoogleDeneme.txt');
//    Free;
//  end;

  LoadHTML(wBrowser, sHTML);

  while wBrowser.ReadyState < READYSTATE_INTERACTIVE do Application.ProcessMessages;
end;

procedure TfrmGoogleMap.btnFindTopkapiClick(Sender: TObject);
var
  Doc : IHTMLDocument2;
  Win : IHTMLWindow2;
begin
  Doc := wBrowser.Document as IHTMLDocument2;
  Win := Doc.parentWindow;

  Win.execScript('GoogleMap.FindGeoCode("Topkap Saray, istanbul");', 'JavaScript');
end;

procedure TfrmGoogleMap.btnAddMarkerClick(Sender: TObject);
var
  Doc : IHTMLDocument2;
  Win : IHTMLWindow2;

  sLat,
  sLng : String;
begin
  Doc := wBrowser.Document as IHTMLDocument2;
  Win := Doc.parentWindow;

  sLat := FloatToStr(MarkerLat);
  sLng := FloatToStr(MarkerLng);

  sLat := StringReplace(sLat, ',', '.', [rfReplaceAll]);
  sLng := StringReplace(sLng, ',', '.', [rfReplaceAll]);

  Win.execScript('GoogleMap.AddMarker(' + sLat + ',' + sLng + ', true);', 'JavaScript');

//  Win.execScript('GoogleMap.AddSmallMapControl();', 'JavaScript');
//  Win.execScript('GoogleMap.AddSmallZoomControl();', 'JavaScript');
end;

procedure TfrmGoogleMap.btnGotoCoordinatClick(Sender: TObject);
var
  Doc : IHTMLDocument2;
  Win : IHTMLWindow2;

  sLat,
  sLng : String;
begin
  Doc := wBrowser.Document as IHTMLDocument2;
  Win := Doc.parentWindow;

  sLat := edtLat.Text;
  sLng := edtLng.Text;

  sLat := StringReplace(sLat, ',', '.', [rfReplaceAll]);
  sLng := StringReplace(sLng, ',', '.', [rfReplaceAll]);

  Win.execScript('GoogleMap.GotoCoordinat(' + sLat + ',' + sLng + ');', 'JavaScript');
  Win.execScript('GoogleMap.AddMarker(' + sLat + ',' + sLng + ', true);', 'JavaScript');
end;

end.
